<?php

namespace Drupal\webform_all_download\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\webform\Entity\WebformSubmission;
use Drupal\webform\Entity\Webform;
use Symfony\Component\HttpFoundation\Response;
use Drupal\webform_all_download\Plugin\Archiver\Zip;

/**
 * Provides route responses for Webform testing.
 */
class WebformDownloadAllController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   *   The entity type manager.
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('file_system'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(FileSystemInterface $file_system, EntityTypeManagerInterface $entityTypeManager) {
    $this->fileSystem = $file_system;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Returns a webform to add a new test submission to a webform.
   *
   * @param string $webform
   *   The webform name.
   * @param int $webform_submission
   *   The webform submission number.
   *
   * @return object
   *   The response object.
   */
  public function download_download($webform, $webform_submission) {
    $filename = $webform . '_' . $webform_submission . '_files';
    $webform = Webform::load($webform);
    $webform_submission = WebformSubmission::load($webform_submission);
    $get_attachments = $webform->getElementsAttachments();
    $submissions = $webform_submission->getData();

    $file = [];
    foreach ($get_attachments as $file_field) {
      $fids = $submissions[$file_field];
      if (!is_array($fids) && !empty($fids) && is_numeric($fids)) {
        $fids = [$fids];
      }
      $file = array_merge($file, $fids);
    }

    $files = [];
    $private_file = $this->fileSystem->realpath('private://') . '/webform_zips';
    $this->fileSystem->prepareDirectory($private_file, FileSystemInterface::CREATE_DIRECTORY);

    $private_file = $private_file . '/' . $webform;
    $file_path = $private_file . '/' . $filename . '.zip';

    if (!empty($file)) {
      $file_zip = new Zip($file_path);
      foreach ($file as $f) {
        $file_obj = $this->entityTypeManager->getStorage('file')->load($f);
        if ($file_obj) {
          $file = $this->fileSystem->realpath($file_obj->getFileUri());
        }
        $file_zip->add($file);
      }

      if ($file_zip instanceof Zip) {
        $file_zip->close();

        $response = new Response();
        $response->headers->set('Content-Type', 'application/octet-stream');
        $response->headers->set('Content-disposition', 'attachment; filename=' . urlencode($filename . '.zip'));
        $response->headers->set('Content-Transfer-Encoding', 'binary');
        $response->setContent(file_get_contents($file_path));
        return $response;
      }
    }
    else {
      $response = new Response();
      return $response;
    }
  }

}
